define(
    [
        "dojo/_base/declare",
        "dojo/ready",
        "dojo/_base/lang",
        "dojo/topic",
        "dijit/_Widget",
        "dijit/_TemplatedMixin",
        "epi/dependency",
        "epi-cms/contentediting/ContentModelServerSync",
        "epi-cms/_ContentContextMixin",
        "epi/shell/widget/_ValueRequiredMixin",
        "opti-recs/codemirror",
        "opti-recs/xml",
        "opti-recs/overlay",
        
        "xstyle/css!opti-recs/codemirror.css",
        "xstyle/css!opti-recs/editor.css",
    ],
    function (
        declare,
        ready,
        lang,
        topic,
        _Widget,
        _TemplatedMixin,
        dependency,
        ContentModelServerSync,
        _ContentContextMixin,
        _ValueRequiredMixin,
        codemirror,
        xml,
        overlay,
        
    ) {
        return declare([_Widget, _TemplatedMixin, _ValueRequiredMixin, _ContentContextMixin],
            {
                templateString: '<div class="dijitInline"><textarea ' +
                    'spellcheck="false" ' +
                    'data-dojo-attach-point="code" ' +
                    'data-dojo-attach-event="onchange:_onChange"' +
                    '></textarea></div>',

                value: null,

                _editorValue: null,

                instance: null,

                editor: null,

                contentDataStore: null,

                syncService: null,

                _setValueAttr: function (value) {
                    this._set('value', value || '');
                    if (this.code) {
                        this.code.value = this.value;
                    }
                    if (this.editor) {
                        this.editor.getDoc().setValue(this.value);
                    }
                },

                constructor: function () {
                    instance = this;
                    this.inherited(arguments);
                
                    this.own(topic.subscribe("opti-recs", function (key) {
                        instance._checkDeliveryKey(key);
                        
                    }, false));

                    if (!this.contentDataStore) {
                        var registry = dependency.resolve("epi.storeregistry");
                        this.contentDataStore = registry.get("epi.cms.contentdata");
                    }

                    if (!this.syncService) {
                        this.syncService = new ContentModelServerSync({
                            contentLink: this.getCurrentContext().id,
                            contentDataStore: this.contentDataStore
                        });
                    }
                    

                    ready(lang.hitch(this, function () {
                        codemirror.defineMode("mustache", function (config, parserConfig) {
                            var mustacheOverlay = {
                                token: function (stream, state) {
                                    var ch;
                                    if (stream.match("{{")) {
                                        while ((ch = stream.next()) != null)
                                            if (ch == "}" && stream.next() == "}") {
                                                stream.eat("}");
                                                return "mustache";
                                            }
                                    }
                                    while (stream.next() != null && !stream.match("{{", false)) { }
                                    return null;
                                }
                            };
                            return codemirror.overlayMode(codemirror.getMode(config, parserConfig.backdrop || "text/html"), mustacheOverlay);
                        });

                        this.set("_editorValue", this.value);
                        this.editor = codemirror.fromTextArea(this.domNode.children[0], { mode: "mustache" });
                        this.editor.setSize(700, 500);;
                        this.editor.on("change", this._onChange);
                    }));
                },
               
                _checkDeliveryKey: function(key) {
                    if (key && this.value && !this.value.includes(`data-api-key="${key}"`)) {
                        this._updatewidgetTemplate(key);
                        this.syncService.saveProperty("recommendationsTemplate", this.value);
                        
                    } else if (!this.value){
                        this._updatewidgetTemplate(key);
                        this.syncService.saveProperty("recommendationsTemplate", this.value);
                    }
                },

                _onChange: function (val) {
                    // summary:
                    //    Raised when the editor's content is changed.
                    //
                    // val:
                    //    The editor's changed value
                    //
                    // tags:
                    //    callback
                    var content = val.getValue();
                    instance._set("value", content);
                    instance.set("_editorValue", content);
                    if (instance.validate()) {
                        instance.onChange(content);
                    }
                },

                _updatewidgetTemplate: function (key) {
                    if (key === null) return;
                    // Summary: Update the region drop down list according to the selected country
                    var template = `<script class="idio-recommendations" type="text/x-mustache" data-api-key="${key}">'
    {{ #content}}
    <div class="idio-recommendation">
        {{ #main_image_url}}
        <a href="{{link_url}}"><img src="{{main_image_url}}?w=150&amp;h=150" class="image" /></a>
        {{/ main_image_url}}

        <h3><a href="{{link_url}}">{{ title }}</a></h3>
        <p>
            {{ abstract }}
            <a href="{{link_url}}">Read more...</a>
        </p>

        <div class="meta">
            <ul class="topics">
                {{ #topics}}
                <li>{{ title }}</li>
                {{/ topics}}
            </ul>

            {{ #source}}
            <span class="source">{{ title }}</span>
            {{/ source}}

            {{ #author}}
            <span class="author">{{ title }}</span>
            {{/ author}}
        </div>
    </div>
    {{/ content}}
</script>`
                    // Clear the current value
                    this._setValueAttr(template);
                    
                }
            });
           
    });